"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.CACHE_FILE_PATH = void 0;

var _chokidar = _interopRequireDefault(require("chokidar"));

var _fs = require("fs");

var _util = require("util");

var _path = require("path");

const readFile = (0, _util.promisify)(_fs.readFile); // We use a CACHE FILE since getStaticPaths runs either in a Worker or a ChildProcess.
// A memoru cache is completely useless in those cases and I honestly ran out of ideas on how to solve this.

const CACHE_FILE_PATH = (0, _path.resolve)(process.cwd(), '.nextein', 'nextein_ipc.cache');
exports.CACHE_FILE_PATH = CACHE_FILE_PATH;

var _default = () => {
  const watcher = _chokidar.default.watch([CACHE_FILE_PATH]);

  let cache;
  let isValid;

  async function build() {
    let path = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : CACHE_FILE_PATH;
    let raw;

    try {
      raw = await readFile(path, {
        encoding: 'utf-8'
      });
      cache = JSON.parse(raw);
      isValid = true;
    } catch (error) {
      // When changing a file too quickly readFile sometimes reports an empty file.
      // so if raw has value then log the error. Swallow when raw is empty
      if (raw) console.error(error);
    }
  }

  watcher.on('change', build);
  return {
    load: async () => build(),
    set: entries => {
      cache = entries;
      isValid = true;
      return entries;
    },
    get: () => cache,
    isValid: () => isValid,
    invalidate: () => {
      isValid = false;
    }
  };
};

exports.default = _default;