"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.compile = compile;

var _config = require("../config");

function compile() {
  const configs = [];
  const sources = [];
  const builders = [];
  const transformers = [];
  const cleaners = [];
  const filters = [];
  const sorters = [];
  const renders = [];

  for (const plugin of (0, _config.plugins)()) {
    const {
      resolved,
      options = {},
      renderer
    } = plugin;

    const {
      config,
      source,
      build,
      transform,
      cleanup,
      filter,
      sort
    } = require(resolved);

    if (config) {
      configs.push(function () {
        for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
          args[_key] = arguments[_key];
        }

        return config(options, ...args);
      });
    }

    if (source) {
      sources.push(function () {
        for (var _len2 = arguments.length, args = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
          args[_key2] = arguments[_key2];
        }

        return source(options, ...args);
      });
    }

    if (build) {
      builders.push(function () {
        for (var _len3 = arguments.length, args = new Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
          args[_key3] = arguments[_key3];
        }

        return build(options, ...args);
      });
    }

    if (transform) {
      transformers.push(function () {
        for (var _len4 = arguments.length, args = new Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
          args[_key4] = arguments[_key4];
        }

        return transform(options, ...args);
      });
    }

    if (cleanup) {
      cleaners.push(function () {
        for (var _len5 = arguments.length, args = new Array(_len5), _key5 = 0; _key5 < _len5; _key5++) {
          args[_key5] = arguments[_key5];
        }

        return cleanup(options, ...args);
      });
    }

    if (filter) {
      filters.push(function () {
        for (var _len6 = arguments.length, args = new Array(_len6), _key6 = 0; _key6 < _len6; _key6++) {
          args[_key6] = arguments[_key6];
        }

        return filter(options, ...args);
      });
    }

    if (sort) {
      sorters.push(function () {
        for (var _len7 = arguments.length, args = new Array(_len7), _key7 = 0; _key7 < _len7; _key7++) {
          args[_key7] = arguments[_key7];
        }

        return sort(options, ...args);
      });
    }

    if (renderer) {
      const {
        render
      } = require(`${resolved}/render`);

      if (render) {
        renders.push(function () {
          for (var _len8 = arguments.length, args = new Array(_len8), _key8 = 0; _key8 < _len8; _key8++) {
            args[_key8] = arguments[_key8];
          }

          return render(options, ...args);
        });
      }
    }
  }

  return {
    configs,
    sources,
    builders,
    transformers,
    cleaners,
    filters,
    sorters,
    renders
  };
}