"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getDefaultPlugins = void 0;
exports.plugins = plugins;
exports.processPlugins = processPlugins;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _resolver = require("./resolver");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { (0, _defineProperty2.default)(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function normalizeString(config) {
  return typeof config === 'string' ? {
    name: config,
    options: {}
  } : config;
}

function normalizeArray(config) {
  if (Array.isArray(config)) {
    const [name, options = {}] = config;
    return {
      name,
      options
    };
  }

  return config;
}

function normalizeObject(config) {
  // At this point we already processed String and Array config. Assuming object.
  return _objectSpread({
    id: config.id || config.name,
    options: config.options || {}
  }, config);
}

function checkForOldMarkdownPlugin(config) {
  const {
    name,
    options = {}
  } = config;

  if (name === 'nextein-plugin-markdown') {
    const {
      entriesDir,
      extension
    } = options;

    if (entriesDir || extension) {
      console.warn('"entriesDir" and "exentension" configs have been removed from markdown plugin.');
      console.warn('Add a "nextein-plugin-source-fs" configuration instead.');
    }
  }

  return config;
}

function resolveSimplifiedNames(config) {
  let {
    name
  } = config;

  if (!name.startsWith('.') && !name.startsWith('@') && !name.startsWith('nextein-plugin-')) {
    name = `nextein-plugin-${name}`;
  }

  return _objectSpread(_objectSpread({}, config), {}, {
    name
  });
}

function createPlugin(options) {
  const resolved = (0, _resolver.resolvePlugin)(options.name);
  const renderer = (0, _resolver.hasRenderer)(resolved);
  return _objectSpread(_objectSpread({}, options), {}, {
    resolved,
    renderer
  });
}

function processDuplicates(prev, curr) {
  return prev.filter(_ref => {
    let {
      id
    } = _ref;
    return id !== curr.id;
  }).concat(curr);
}

function processPlugins() {
  let nexteinPlugins = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
  const config = nexteinPlugins.map(normalizeString).map(normalizeArray).map(normalizeObject).map(checkForOldMarkdownPlugin).map(resolveSimplifiedNames).map(createPlugin).reduce(processDuplicates, []);
  process.env.__NEXTEIN_PLUGIN_CFG = JSON.stringify(config);
  return config;
}

function plugins() {
  return JSON.parse(process.env.__NEXTEIN_PLUGIN_CFG || '{}');
} // TODO


const getDefaultPlugins = () => [['nextein-plugin-source-fs', {
  path: 'posts'
}], 'nextein-plugin-markdown', 'nextein-plugin-filter-unpublished', 'nextein-plugin-sort-by-date'];

exports.getDefaultPlugins = getDefaultPlugins;