"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.source = source;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _assert = _interopRequireDefault(require("assert"));

var _chokidar = _interopRequireDefault(require("chokidar"));

var _util = require("util");

var _fs = require("fs");

var _path = require("path");

var _mime = _interopRequireDefault(require("mime"));

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { (0, _defineProperty2.default)(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

const asyncReadFile = (0, _util.promisify)(_fs.readFile);
const DEFAULT_IGNORES = ['**/*.un~', '**/.DS_Store', '**/.gitignore', '**/.npmignore', '**/.babelrc', '**/yarn.lock', '**/bower_components', '**/node_modules', '**/package-lock.json'];
const DATE_IN_FILE_REGEX = /^(\d{4}-\d{2}-\d{2})-(.+)$/;
const DATE_MATCH_INDEX = 1;
const NAME_MATCH_INDEX = 2;
/**
 * Create a buildOptions
 * @param {*} filePath
 * @param {*} basePath
 * @param {Object} data
 */

function buildOptions(filePath, basePath, data) {
  const fileName = (0, _path.basename)(filePath, (0, _path.extname)(filePath));
  const match = fileName.match(DATE_IN_FILE_REGEX);
  const {
    birthtime
  } = (0, _fs.statSync)(filePath);
  const path = (0, _path.relative)(basePath, (0, _path.dirname)(filePath));
  const name = match ? match[NAME_MATCH_INDEX] : fileName;
  const createdOn = JSON.stringify(match ? new Date(match[DATE_MATCH_INDEX]) : birthtime);
  return {
    filePath,
    path,
    name,
    mimeType: _mime.default.getType(filePath),
    createdOn,
    extra: _objectSpread({}, data),

    async load() {
      return asyncReadFile(filePath, {
        encoding: 'utf-8'
      });
    }

  };
}
/**
 * Source Plugin to read files from File System
 * @param {Object} options
 * @param {String} options.path path where entries will be loaded.
 * @param {String} options.includes
 * @param {Array<String>} options.ignore
 * @param {Object} data default data added to extra field for each entry.
 * @param {Object} action
 * @param {function(buildOptions)} action.build
 * @param {function({ filePath })} action.remove
 */


async function source() {
  let {
    path: pathOptions,
    ignore,
    includes = '**/*.*',
    data = {}
  } = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  let {
    build,
    remove
  } = arguments.length > 1 ? arguments[1] : undefined;
  (0, _assert.default)(pathOptions, 'The path is required in source-filesystem plugin configuration.'); // Make sure path is absolute.

  const path = !(0, _path.isAbsolute)(pathOptions) ? (0, _path.resolve)(process.cwd(), pathOptions) : pathOptions; // Use path with includes to create a glob

  const watcher = _chokidar.default.watch((0, _path.normalize)(`${path}/${includes}`), {
    ignored: [...DEFAULT_IGNORES, ...(ignore || [])]
  }); // We'll enqueue the first batch. Then after ready we will process updates one by one.


  let batch = true;
  const queue = []; // add file

  watcher.on('add', async filePath => {
    batch ? queue.push(filePath) : await build(buildOptions(filePath, path, data));
  }); // modify file

  watcher.on('change', async filePath => {
    await build(buildOptions(filePath, path, data));
  }); // remove file

  watcher.on('unlink', async filePath => {
    await remove({
      filePath
    });
  }); // TODO: do we need to process `addDir`, `unlinkDir`?

  return new Promise(resolve => {
    watcher.on('ready', async () => {
      for (const filePath of queue) {
        await build(buildOptions(filePath, path, data));
      }

      batch = false;
      resolve();
    });
  });
}