"use strict";

var _fs = require("fs");

var _path = require("path");

var _util = require("util");

var _worker_threads = require("worker_threads");

var _cache = require("../entries/cache");

var _bootstrap = require("./bootstrap");

const writeFile = (0, _util.promisify)(_fs.writeFile);
const mkdir = (0, _util.promisify)(_fs.mkdir);

async function updateTmpFile(data) {
  try {
    await mkdir((0, _path.dirname)(_cache.CACHE_FILE_PATH), {
      recursive: true
    });
    await writeFile(_cache.CACHE_FILE_PATH, JSON.stringify(data), {
      encoding: 'utf-8',
      flag: 'w+'
    });
  } catch (error) {
    console.error('Error writing cache.', error);
  }
}

if (_worker_threads.isMainThread) {
  let worker;

  module.exports.runProcessWorker = function () {
    worker = new _worker_threads.Worker(__filename);
    worker.on('message', updateTmpFile);
  };
} else {
  // run code
  (0, _bootstrap.subscribe)(data => _worker_threads.parentPort.postMessage(data));
  (0, _bootstrap.run)().then(data => _worker_threads.parentPort.postMessage(data));
}